#!/usr/bin/env python
# by esde 2018
# updated for better G support, 27jan2019

import sys, json, struct

def gen_model(mod):
	out = 0

	if 'A' in mod:
		out += 0x80

	if 'D' in mod:
		out += 0x40

	if 'G' in mod:
		out += 0x20

	return [out, 0]

def gen_mux(mux):
	if mux == "18/25":
		return [1]
	elif mux == "27/16":
		return [2]
	else:
		return [0]

def gen_clock(clk):
	vals = {'rising': 1, 'falling': 2, 'edge': 3}
	out = {'J': 0, 'K': 0, 'L': 0}

	for k,v in clk.viewitems():
		if vals.has_key(v):
			out[k] = vals[v]
		else:
			out[k] = 0

	return [out['J'], out['K'], out['L']]

def gen_threshold(threshold, suffix):
	if suffix == 'AD':
		return [0]*15
	else:
		return [0]*21

def gen_polarity(pol):
	pass

# POD: width, shift
podmap_AD = {
	'POD0': [8,  0],
	'POD1': [8,  8],
	'POD2': [9, 16],
	'POD3': [9, 25],
	'POD4': [9, 34]
}

podmap_G = {
	'POD1': [8,   0],
	'POD2': [9,   8],
	'POD3': [9,  17],
	'POD4': [9,  26],
	'POD5': [10, 35],
	'POD6': [10, 45],
    'POD7': [10, 55]
}

def gen_mask(podmask, podmap):
	mask = 0
	calc = {}

	gerd = dict(podmask)

	pn, ms = gerd.popitem()

	for i,n in enumerate(ms[::-1]):
		if n == '*':
			mask |= (1 << i)

	return mask << podmap[pn][1]

def gen_assignment(pod, suffix):
	def to_bytes(val, len):
		return ('%%0%dx' % (len << 1) % val).decode('hex')[-len:]

	mask = 0
	for n in pod['members']:
		if suffix == 'AD':
			mask |= gen_mask(n, podmap_AD)
		else:
			mask |= gen_mask(n, podmap_G)

	if suffix == 'AD':
		rv = list(bytearray(to_bytes(mask, 6)))
	else:
		rv = list(bytearray(to_bytes(mask, 9)))

	# Polarity setting
	if pod['polarity'] == '-':
		pol = 1
	else:
		pol = 0

	# Polarity, fixed byte, pod mask bytes
	return [pol, 0x03] + rv

def gen_userbase(names):
	return list(bytearray("".join([n.ljust(5) for n in names]), 'utf-8'))

#
# Start
#
if len(sys.argv) != 3:
	print 'usage: genprobe.py [in.json] [out.bin]'
	sys.exit(1)

fp = open(sys.argv[1], 'r')
dat = fp.read()
fp.close()

obj = json.loads(dat)

# Assemble the parts...
out = []
out += gen_model(obj['model_flag'])
out += gen_mux(obj['mux'])
out += gen_clock(obj['master_clock'])
out += gen_clock(obj['slave_clock'])
out += gen_threshold(obj['AD']['threshold'], 'AD')
out += gen_assignment(obj['AD']['assignments']['ADDR'], 'AD')
out += gen_assignment(obj['AD']['assignments']['DATA'], 'AD')
out += gen_assignment(obj['AD']['assignments']['STAT'], 'AD')
out += gen_assignment(obj['AD']['assignments']['EXTRA'], 'AD')
out += gen_userbase(obj['user_base_1'])
out += gen_userbase(obj['user_base_2'])

#
# Genrating the G suffix version would be mostly more of the same...
# The differences would be:
#   no model flag
#   threshold area is 6 bytes longer
#   probe masks are 9 bytes instead of 6...however they might be shifted strangely
#
# XXX: For now, going to generate an empty but valid one for G models.
#
out += gen_mux(obj['mux'])
out += gen_clock(obj['master_clock'])
out += gen_clock(obj['slave_clock'])
out += gen_threshold(obj['G']['threshold'], 'G')
out += gen_assignment(obj['G']['assignments']['ADDR'], 'G')
out += gen_assignment(obj['G']['assignments']['DATA'], 'G')
out += gen_assignment(obj['G']['assignments']['STAT'], 'G')
out += gen_assignment(obj['G']['assignments']['EXTRA'], 'G')
out += gen_userbase(obj['user_base_1'])
out += gen_userbase(obj['user_base_2'])

# Trailing padding - seems to be present in the samples, and does round out
# to a nice 512 bytes.
out += [0] * 0x40

print "Generated probe config using %s, %d bytes" % (sys.argv[1], len(out))

fp = open(sys.argv[2], 'wb')
fp.write("".join([chr(n) for n in out]))
fp.close()
